#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-
"""
Simply Draw for Python.
See arminstraub.de for more information.

Copyright (C) 2005 by Armin Straub <linux@arminstraub.de>
"""

"""
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
"""

import math
from simplydraw import style, canvas, translate, stroked, filled, \
        topt, halign, vshift, box, unit


# Add canvas.point().

dot = style._circlesymbol
cross = style._crosssymbol
plus = style._plussymbol

# These identifiers might also be useful as paths. So I will let them
# commented out until I really need them.
#square = style._squaresymbol
#triangle = style._trianglesymbol
#diamond = style._diamondsymbol

def _point(self, x, y, type=dot, size=0.2*unit.v_cm, attrs=[]):
    """Make a single dot.
    
    Draws a single point. You can specify its type (dot, cross,
    plus currently) via type.
    size gives the size of the point, but note that a dot is in
    fact drawn smaller, so that "the sizes match" between the
    types. You may specify attributes as usually with attrs"""
    if type == dot:
        size = size/2
        attrs = [filled] + attrs
    else:
        attrs = [stroked] + attrs
    type(self, topt(x), topt(y), topt(size), attrs)

canvas.point = _point


# Add canvas.label().

def _label(self, x, y, text, attrs=[], angle=45, labelsep=.2*unit.v_cm):
    """Places a label.
    
    This function additionally accepts the keyword arguments
    labelsep and angle. It places some text in the direction given
    by angle (default: 45) and the distance (default: .06) given
    by labelsep.
    The angle can also be given via a string as 'ur' (45) or
    'l' (180) and so on just as the pstricks function uput.
    Instead of text you can also pass a canvas object."""
    angles = {'r':0, 'ur':45, 'u':90, 'ul':135, 'l':180,
        'dl':225, 'd':270, 'dr':315}
    if angle in angles:
        angle = angles[angle]
    a = math.pi/180*angle

    if isinstance(text, canvas):
        c = text
        w, h = c.bbox().width(), c.bbox().height()
        b = box.rect(-w/2, -h/2, w, h, relcenter=(.5, .5))
        cx, cy = c.bbox().center()
        x, y = x-cx, y-cy
    else:
        c = self.texrunner.text(0, 0, text,
                [halign.center, vshift.mathaxis] + attrs)
        b = c
        attrs = []
    # linealignvector() is possible as well (may be better for objects
    # much bigger than labelsep).
    dx, dy = b.circlealignvector(labelsep, math.cos(a), math.sin(a))
    self.insert(c, attrs + [translate(x+dx, y+dy)])

canvas.label = _label


# Add canvas.stack()

def _stack(self, items, spacing=(.4*unit.v_cm, .4*unit.v_cm)):
    """Stacks canvas objects as given in the 2d array.
    
    Can be used to stack e.g. graphs in a 2x2 field using
    stack([ [g1, g2], [g3, g4]]).
    spacing sets the space left between different objects."""
    hspace, vspace, y = spacing[0], spacing[1], 0
    for row in items:
        x, dy = 0, 0
        for obj in row:
            box = obj.bbox()
            self.insert(obj, [translate(x - box.left(), y - box.bottom())])
            x += hspace + box.width()
            dy = max(dy, box.height())
        y -= vspace + dy

canvas.stack = _stack
