#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-
"""
Simply Draw for Python.
See arminstraub.de for more information.

Copyright (C) 2005 by Armin Straub <linux@arminstraub.de>
"""

"""
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
"""

from simplydraw import data, style, earrow, ticklength, graphxy, \
        attr
from pyx import path as _path


# An alternative to graphxy.

def mathxy(*args, **kwargs):
    """A more mathematical graph.
    
    Wrapper for graphxy that looks (for my purpose) more mathematical.
    Second axes are killed. The axes meet in (0,0) and have the
    style of an arrow."""
    # No second axis
    if not 'x2' in kwargs: kwargs['x2'] = None
    if not 'y2' in kwargs: kwargs['y2'] = None
    # Axis intersect in (0,0)
    if not 'xaxisat' in kwargs: kwargs['xaxisat'] = 0
    if not 'yaxisat' in kwargs: kwargs['yaxisat'] = 0
    # Add arrow and complete ticks
    for axis in ['x', 'y']:
        if axis in kwargs:
            p = kwargs[axis].painter
            if p.outerticklength == None:
                p.outerticklength = ticklength.normal
            p.basepathattrs = p.basepathattrs + [earrow.normal]
            # Not the same as
            # p.basepathattrs += [earrow.normal]
            # because this changes but not replaces the object basepathattrs.
            # So any axis with default value will get this changed.
    return graphxy(*args, **kwargs)


# Enhance data.function() and data.paramfunction()

def _function(*args, **kwargs):
    """Enhanced version of data.function() from PyX core.
    
    You can pass a real Python function as expression."""
    # The first argument should be an expression (either string or Python).
    if args:
        expression = args[0]
    else:
        expression = kwargs['expression']
    if callable(expression):
        # We just have to replace the Python expression by its string
        # and set the the context so that it can be found.
        string = "y(x)=expression(x)"
        if args:
            args = (string,) + args[1:]
        else:
            kwargs['expression'] = string
        kwargs['context'] = locals()
    return _origdatafunction(*args, **kwargs)

def _paramfunction(*args, **kwargs):
    """Enhanced version of data.paramfunction() from PyX core.
    
    You can pass a real Python expression, too, while
    omitting the varname parameter."""
    # This is a bad hack. We are either called as (min, max, expression)
    # or as (varname, min, max, expression).
    # So expression might also be max... but not callable then...
    if 'expression' in kwargs:
        expression = kwargs['expression']
    else:
        expression = args[2]
    if callable(expression):
        # Replace expression by its string, add varname parameter and set
        # context appropriate.
        string = "x,y=expression(t)"
        if 'expression' in kwargs:
            kwargs['expression'] = string
        else:
            args = args[0:2] + (string,) + args[3:]
        args = ("t",) + args
        kwargs['context'] = locals()
    return _origdataparamfunction(*args, **kwargs)

_origdatafunction = data.function
data.function = _function

_origdataparamfunction = data.paramfunction
data.paramfunction = _paramfunction


# Enhance data.list().

def _list(*args, **kwargs):
    """Enhanced version of data.list() from PyX core.
    
    Enhances data.list which is usually called (by me) as
    data.list(points, x=1, y=2). Now you can call it just like
    data.list(points)."""
    if kwargs == {}:
        kwargs = {'x': 1, 'y': 2}
    return _origdatalist(*args, **kwargs)

_origdatalist = data.list
data.list = _list


# Add graph style style.steps().

class _steps(style._styleneedingpointpos):
    """A graph style that is similar to line but draws steps.
    
    The line graph style is nice for drawing continuous functions, but not
    optimal for functions that are e.g. known to be step functions. This simple
    style will only draw steps.
    I should think about a way to detect arbitrary points of discontinuity, but
    that's not what I needed for now..."""

    needsdata = ["vpos", "vposmissing", "vposvalid"]

    defaultlineattrs = []

    def __init__(self, lineattrs=[], epsilon=1e-10):
        self.lineattrs = lineattrs
        self.epsilon = epsilon

    def selectstyle(self, privatedata, sharedata, graph, selectindex, selecttotal):
        if self.lineattrs is not None:
            privatedata.lineattrs = attr.selectattrs(self.defaultlineattrs + self.lineattrs, selectindex, selecttotal)
        else:
            privatedata.lineattrs = None

    def initdrawpoints(self, privatedata, sharedata, graph):
        privatedata.path = _path.path()
        privatedata.linefirst = None
        privatedata.linelast = None

    def addpendingline(self, privatedata, sharedata):
        if not privatedata.linefirst is None and not privatedata.linelast is None:
            privatedata.path.append(_path.moveto_pt(*privatedata.linefirst))
            privatedata.path.append(_path.lineto_pt(*privatedata.linelast))
        privatedata.linefirst = None
        privatedata.linelast = None

    def drawpoint(self, privatedata, sharedata, graph, point):
        if sharedata.vposvalid:
            newpoint = graph.vpos_pt(*sharedata.vpos)
            if privatedata.linefirst is None:
                # start new line
                privatedata.linefirst = newpoint
            else:
                if abs(newpoint[1] - privatedata.linefirst[1]) <= self.epsilon:
                    # continue line
                    privatedata.linelast = newpoint
                else:
                    # finish line and start a new one
                    self.addpendingline(privatedata, sharedata)
                    privatedata.linefirst = newpoint
        else:
            self.addpendingline(privatedata, sharedata)

    def donedrawpoints(self, privatedata, sharedata, graph):
        self.addpendingline(privatedata, sharedata)
        if privatedata.lineattrs is not None and len(privatedata.path):
            graph.stroke(privatedata.path, privatedata.lineattrs)

    def key_pt(self, privatedata, sharedata, graph, x_pt, y_pt, width_pt, height_pt):
        raise RuntimeError("Style currently doesn't provide a graph key")

style.steps = _steps
